--
-- $Id: PathUtils.hs,v 1.3 2005/12/04 05:31:59 aamine Exp $
--
-- Copyright (c) 2003-2005 Minero Aoki
--
-- This program is free software.
-- You can distribute/modify this program under the terms of
-- the GNU LGPL, Lesser General Public License version 2.1.
-- For details of the GNU LPGL, see the file "COPYING".
--

module PathUtils
    (concatPath, joinPath, basename, dirname, components, ancestors)
where

import Data.List (intersperse)

concatPath :: [String] -> String
concatPath [] = "/"
concatPath [""] = "/"
concatPath cs = join "/" cs

joinPath :: String -> String -> String
joinPath a b
    | null a        = '/' : b
    | last a == '/' = a ++ b
    | otherwise     = a ++ "/" ++ b

basename :: String -> String
basename "/"  = "/"
basename path = last (components path)

ancestors :: String -> [String]
ancestors path = case scanl1 (joinPath) $ components path of
                   ("":xs) -> reverse ("/":xs)
                   xs      -> reverse xs

components :: String -> [String]
components = split '/'

dirname :: String -> String
dirname "/" = "/"
dirname path = case components path of
                 []  -> ""
                 [_] -> "."
                 cs  -> concatPath (init cs)

split :: Char -> String -> [String]
split _ [] = []
split sep str = word : split sep cont
  where
    (word, cont') = break (==sep) str
    cont = case cont' of
             []     -> ""
             (c:cs) -> cs

join :: String -> [String] -> String
join sep = concat . intersperse sep
